
extern crate proc_macro;

use proc_macro::{TokenStream, TokenTree};
use quote::quote;
use regex::Regex;
use syn::{parse_macro_input, Expr, Ident, ItemStruct};

#[proc_macro_attribute]
pub fn advent_day(attrs: TokenStream, item: TokenStream, ) -> TokenStream {
    let input = parse_macro_input!(item as ItemStruct);
    let ident = &input.ident;

    let mut parts = vec![];
    for token in attrs.into_iter() {
        match token {
            TokenTree::Ident(ident) => parts.push(ident.to_string()),
            _ => continue
        }
    }

    let call1 = format!("{}::new()", &parts[0]);
    let call2 = format!("{}::new()", &parts[1]);
    let part1: Expr = syn::parse_str(&call1.as_str()).unwrap();
    let part2: Expr = syn::parse_str(&call2.as_str()).unwrap();

    
    let ident_part1: Ident = syn::parse_str(&parts[0].as_str()).unwrap();
    let ident_part2: Ident = syn::parse_str(&parts[1].as_str()).unwrap();

    let re = Regex::new(r"(\d+)").unwrap();
    let hay = input.ident.to_string();
    let (_, [day_nr]) = re.captures(hay.as_str()).unwrap().extract();
    let input_str = format!("resources/input{}.txt", day_nr);

    let output = quote! {

        impl AdventDayPart1 for #ident_part1 {
            fn read_input(&mut self) -> Result<(), AdventError> {
                read_input_part1(self, Path::new(#input_str))
            }

            fn solve(&mut self) -> Result<u64, AdventError> {
                solve_part1(self)
            }
        }

        impl AdventDayPart2 for #ident_part2 {
            fn read_input(&mut self) -> Result<(), AdventError> {
                read_input_part2(self, Path::new(#input_str))
            }

            fn solve(&mut self) -> Result<u64, AdventError> {
                solve_part2(self)
            }
        }

        pub struct #ident {
            part1: Box::<dyn AdventDayPart1>,
            part2: Box::<dyn AdventDayPart2>
        }

        impl #ident {
            pub fn new() -> Self {
                Self {
                    part1: Box::new(#part1),
                    part2: Box::new(#part2)
                }
            }
        }          

        impl AdventDay for #ident {
            fn puzzle1(&mut self) -> Result<u64, AdventError> {
                self.part1.read_input()?;
                self.part1.solve()
            }

            fn puzzle2(&mut self) -> Result<u64, AdventError> {
                self.part2.read_input()?;
                self.part2.solve()
            }
        }
    };
    output.into()

}
