use std::{path::Path, str::FromStr};

use crate::input::read_into_vec;

use super::*;

struct Report(Vec<i32>);

impl FromStr for Report {
    type Err = AdventError;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        Ok(Report(s.split_whitespace()
            .map(|s| -> Result<i32, AdventError> { Ok(s.parse::<i32>()?)})
            .collect::<Result<Vec<i32>, AdventError>>()?
            ))
    }
}

impl Report {
    fn safe1(&self) -> bool {
        let diff_vec: Vec<i32> = self.0.iter().zip(self.0.iter().skip(1)).map(|(l, r)| l - r).collect();
        diff_vec.iter().all(|x| {*x >= -3 && *x <= -1}) || diff_vec.iter().all(|x| *x >= 1 && *x <= 3) 
    }

    fn safe2(&self) -> bool {
        let diff_vec: Vec<i32> = self.0.iter().zip(self.0.iter().skip(1)).map(|(l, r)| l - r).collect();
        if diff_vec.iter().all(|x| {*x >= -3 && *x <= -1}) || diff_vec.iter().all(|x| *x >= 1 && *x <= 3) {
            return true;
        }

        for i in 0..(self.0.len()) {
            let mut new_vec = self.0.clone();
            new_vec.remove(i);
            let new_diff_vec : Vec<i32> = new_vec.iter().zip(new_vec.iter().skip(1)).map(|(l, r)| l - r).collect();
            if new_diff_vec.iter().all(|x| {*x >= -3 && *x <= -1}) || new_diff_vec.iter().all(|x| *x >= 1 && *x <= 3) {
                return true;
            }
        }
        return false;
    }
}

struct Day2Part1 {
    reports: Vec<Report>
}

struct Day2Part2 {
    reports: Vec<Report>
}

pub struct Day2 {
    part1: Day2Part1,
    part2: Day2Part2
}

impl Day2 {
    pub fn new() -> Self {
        Day2 {
            part1: Day2Part1::new(),
            part2: Day2Part2::new()
        }
    }
}

impl AdventDay for Day2 {
    fn puzzle1(&mut self) -> Result<u64, AdventError> {
        self.part1.read_input()?;
        self.part1.solve()
    }

    fn puzzle2(&mut self) -> Result<u64, AdventError> {
        self.part2.read_input()?;
        self.part2.solve()
    }
}

impl Day2Part1 {
    fn new() -> Self {
        Day2Part1 {
            reports: vec!()
        }
    }
}

impl AdventDayPart1 for Day2Part1 {
    fn read_input(&mut self) -> Result<(),AdventError> {
        Ok(self.reports = read_into_vec::<Report>(Path::new("resources/input2.txt"))?)
    }

    fn solve(&mut self) -> Result<u64, AdventError> {
        Ok(self.reports.iter().filter(|x| x.safe1()).count().try_into().unwrap())
    }
}

impl Day2Part2 {
    fn new() -> Self {
        Day2Part2 {
            reports: vec!()
        }
    }
}

impl AdventDayPart2 for Day2Part2 {
    fn read_input(&mut self) -> Result<(),AdventError> {
        Ok(self.reports = read_into_vec::<Report>(Path::new("resources/input2.txt"))?)
    }

    fn solve(&mut self) -> Result<u64, AdventError> {
        Ok(self.reports.iter().filter(|x| x.safe2()).count().try_into().unwrap())
    }
}
