use std::path::Path;

use regex::Regex;

use crate::input::get_input;

use super::*;

struct Mul {
    l: i32,
    r: i32,
}

struct Day3Part1(Vec<Mul>);

impl Day3Part1 {
    fn new() -> Self {
        Day3Part1(vec![])
    }
}

impl AdventDayPart1 for Day3Part1 {
    fn read_input(&mut self) -> Result<(),AdventError> {
        let hay = get_input(Path::new("resources/input3.txt"));
        let re = Regex::new(r"mul\((\d+),(\d+)\)").unwrap();
        for (_, [l, r]) in re.captures_iter(&hay).map(|c| c.extract())
        {
            self.0.push(Mul {l: l.parse::<i32>()?, r: r.parse::<i32>()?})
        }
        Ok(())
    }

    fn solve(&mut self) -> Result<u64, AdventError> {
        Ok(self.0.iter().fold(Ok(0), |acc, mul| -> Result<u64,AdventError> {Ok(acc? + TryInto::<u64>::try_into(mul.l * mul.r)?)})?)
    }
}

struct Day3Part2(Vec<Mul>);

impl Day3Part2 {
    fn new() -> Self {
        Day3Part2(vec![])
    }
}

impl AdventDayPart2 for Day3Part2 {
    fn read_input(&mut self) -> Result<(),AdventError> {
        let hay = get_input(Path::new("resources/input3.txt"));
        let re = Regex::new(r"mul\((\d+),(\d+)\)|don\'t\(\)|do\(\)").unwrap();
        let mut state = 1;
        for capture in re.captures_iter(&hay)
        {
            let str = capture.get(0).unwrap().as_str();
            if str == "do()" {
                state = 1;
                continue;
            }
            if str == "don't()" {
                state = 0;
                continue;
            }
            if state == 1 {
                let l = capture.get(1).unwrap().as_str();
                let r = capture.get(2).unwrap().as_str();
                self.0.push(Mul {l: l.parse::<i32>()?, r: r.parse::<i32>()?})
            }
        }
        Ok(())
    }

    fn solve(&mut self) -> Result<u64, AdventError> {
        Ok(self.0.iter().fold(Ok(0), |acc, mul| -> Result<u64,AdventError> {Ok(acc? + TryInto::<u64>::try_into(mul.l * mul.r)?)})?)
    }
}

pub struct Day3 {
    part1: Day3Part1,
    part2: Day3Part2
}

impl Day3 {
    pub fn new() -> Self {
        Day3 {
            part1: Day3Part1::new(),
            part2: Day3Part2::new()
        }
    }
}

impl AdventDay for Day3 {
    fn puzzle1(&mut self) -> Result<u64, crate::error::AdventError> {
        self.part1.read_input()?;
        self.part1.solve()
    }

    fn puzzle2(&mut self) -> Result<u64, crate::error::AdventError> {
        self.part2.read_input()?;
        self.part2.solve()
    }
}