use std::path::Path;

use crate::input::read_into_vec;

use super::*;

use advent_derive::advent_day;
use regex::Regex;

struct Day4Part1 {
    text: Vec<String>
}

impl Day4Part1 {
    fn new() -> Self {
        Day4Part1{
            text: vec![]
        }
    }
}

struct Day4Part2 {
    text: Vec<String>,
    grid_size: usize
}

impl Day4Part2 {
    fn new() -> Self {
        Day4Part2{
            text: vec![],
            grid_size: 0usize
        }
    }
}

fn read_input_part1(part1: &mut Day4Part1, path: &Path) -> Result<(), AdventError> {
    part1.text = read_into_vec(path)?;
    Ok(())
}

fn solve_part1(part1: &mut Day4Part1) -> Result<u64, AdventError> {
    let lr = part1.text.clone();
    let mut ud = vec![];
    let grid_size = part1.text.len();

    for i in 0usize .. grid_size {
        ud.push(part1.text.iter().map(|s| {
            s.chars().nth(i).unwrap()
        }).collect::<String>());
    }

    let mut lurd = vec![];
    for i in 0usize .. grid_size {
        let mut j = i;
        
        lurd.push(part1.text.iter().map(|s| {
            let c;
            if j >= grid_size {
                c = ' ';
            } else {
                c = s.chars().nth(j).unwrap();
            }
            j += 1;
            c
        }).collect::<String>());
    }
    for i in 1usize .. grid_size {
        let mut j = i;
        
        lurd.push(ud.iter().map(|s| {
            let c;
            if j >= grid_size {
                c = ' ';
            } else {
                c = s.chars().nth(j).unwrap();
            }
            j += 1;
            c
        }).collect::<String>());
    }

    let mut ruld = vec![];
    for i in 0usize .. grid_size {
        let mut j = i;
        
        ruld.push(lr.iter().map(|s| {
            let c;
            if j >= grid_size {
                c = ' ';
            } else {
                c = s.chars().rev().nth(j).unwrap();
            }
            j += 1;
            c
        }).collect::<String>());
    }
    for i in 1usize .. grid_size {
        let mut j = 0;
        ruld.push(lr.iter().skip(i).map(|s| {
            let c;
            if j >= grid_size {
                c = ' ';
            } else {
                c = s.chars().rev().nth(j).unwrap();
            }
            j += 1;
            c
        }).collect::<String>());
    }

    let xmas_r = Regex::new("XMAS")?;
    let samx_r = Regex::new("SAMX")?;

    // println!("input:\n{:?}", part1.text);
    // println!("lr:\n{:?}", lr);
    // println!("up:\n{:?}", ud);
    // println!("lurd:\n{:?}", lurd);
    // println!("ruld:\n{:?}", ruld);

    Ok(
        lr.iter().chain(ud.iter()).chain(lurd.iter()).chain(ruld.iter()).fold(Ok(0), |sum, s| -> Result<u64, AdventError> {
            let mut sum = sum?;
            sum += xmas_r.find_iter(s.as_str()).count() as u64;
            sum += samx_r.find_iter(s.as_str()).count() as u64;
            Ok(sum)    
        })?
    )

}

fn read_input_part2(part2: &mut Day4Part2, path: &Path) -> Result<(), AdventError> {
    part2.text = read_into_vec(path)?;
    part2.grid_size = part2.text.len();
    Ok(())
}

fn is_m(part2: &mut Day4Part2, (x, y): (usize, usize)) -> bool {
    part2.text.iter().skip(y).next().unwrap().chars().nth(x).unwrap() == 'M'
}

fn is_s(part2: &mut Day4Part2, (x, y): (usize, usize)) -> bool {
    part2.text.iter().skip(y).next().unwrap().chars().nth(x).unwrap() == 'S'
}

fn is_xmas(part2: &mut Day4Part2, (x, y): (usize, usize)) -> bool {
    if x == 0 || x == part2.grid_size - 1 || y == 0 || y == part2.grid_size - 1 {
        return false;
    }

    let lr_mas = is_m(part2, (x-1, y-1)) && is_s(part2, (x+1, y+1)) || is_m(part2, (x+1, y+1)) && is_s(part2, (x-1, y-1));
    let rl_mas = is_m(part2, (x+1, y-1)) && is_s(part2, (x-1, y+1)) || is_m(part2, (x-1, y+1)) && is_s(part2, (x+1, y-1));

    lr_mas && rl_mas
}


fn solve_part2(part2: &mut Day4Part2) -> Result<u64, AdventError> {
    let text = part2.text.clone();
    Ok(
        text.iter().enumerate().fold(0, |acc, (y, s)| {
            acc + s.chars().enumerate().fold(0, |acc, (x, c)| {
                if c == 'A' && is_xmas(part2, (x, y)) {
                    acc + 1
                } else {
                    acc
                }
            })
        })
    )
}



#[advent_day(Day4Part1, Day4Part2)]
struct Day4;



