use std::path::Path;

use advent_derive::advent_day;
use itertools::Itertools;
use regex::Regex;

use crate::input::get_lines;

use super::*;

#[derive(Debug)]
struct Order {
    left: u32,
    right: u32,
}

struct Day5Part1 {
    ordering: Vec<Order>,
    queues: Vec<Vec<u32>>
}

impl Day5Part1 {
    fn new() -> Self {
        Self {
            ordering: vec![],
            queues: vec![vec![]]
        }
    }
}

struct Day5Part2 {}

impl Day5Part2 {
    fn new() -> Self {
        Self{}
    }
}

fn read_input_part1(part1: &mut Day5Part1, path: &Path) -> Result<(), AdventError> {
    let mut lines = get_lines(path);
    let order_regex = Regex::new(r"(\d+)\|(\d+)").unwrap();
    part1.ordering = lines.by_ref().take_while(|s| {
        s.as_ref().unwrap().trim() != ""
    }).map(|s| -> Order {
        let s = s.as_ref().unwrap().as_str();
        let (_, [l, r]) = order_regex.captures(s).unwrap().extract();
        Order {
            left: l.parse::<u32>().unwrap(),
            right: r.parse::<u32>().unwrap()
        }
    }).collect::<Vec<Order>>();
    part1.queues = lines.map(|s| {
        let s = s.as_ref().unwrap();
        let print_item_regex = Regex::new(r"\d+").unwrap();
        print_item_regex.find_iter(s.as_str()).map(|m| {
            m.as_str().parse::<u32>().unwrap()
        }).collect::<Vec<u32>>()
    }).collect::<Vec<Vec<u32>>>();
    Ok(())
}

fn is_correct(part1: &mut Day5Part1, q: Vec<u32>) -> bool {
    q.iter().enumerate().all(|(index, item)| {
        part1.ordering.iter().filter(|order| {
            order.left == *item 
        }).all(|order| {
            index < q.iter().find_position(|x| { **x == order.right } ).or(Some((usize::MAX, &0))).unwrap().0
        })
    })
}

fn solve_part1(part1: &mut Day5Part1) -> Result<u64, AdventError> {
    Ok(part1.queues.clone().into_iter().filter(
        |q| {
            is_correct(part1, q.clone())
        }
    ).fold(0u64, | acc, item| {
        acc + item[item.len() / 2] as u64
    }))
}

fn read_input_part2(part2: &mut Day5Part2, path: &Path) -> Result<(), AdventError> {
    Ok(())
}


fn solve_part2(part2: &mut Day5Part2) -> Result<u64, AdventError> {
    Ok(0)
}


#[advent_day(Day5Part1, Day5Part2)]
struct Day5;